import sys
import typer
from .core import PipConf

app = typer.Typer(add_completion=False)


@app.command()
def add(extra: bool = False, username: str = '', password: str = '', repo: str = '', conf: str = ''):
    pc = PipConf(conf or None)
    assert repo
    if extra:
        pc.add_extra_url(repo, username, password)
    else:
        pc.set_index_url(repo, username, password)
    pc.save()


@app.command()
def remove(repo: str = '', conf: str = ''):
    pc = PipConf(conf or None)
    if repo:
        ret = pc.remove_extra_url(repo)
        if ret:
            sys.stdout.write('removed: %s' % repo)
            return 0
        else:
            sys.stderr.write(f'No repo named {repo}')
            return 1
    else:
        pc.clear_extra_url()
        return 0



import sys
import typer
from .core import PyPiRC

app = typer.Typer(add_completion=False)


@app.command()
def add(username: str = '', password: str = '', repo: str = '', server: str = '', rc: str = ''):
    ppr = PyPiRC(rc or None)
    assert server
    if ppr.servers:
        entry = ppr.servers.get(server, {})
    else:
        entry = {}
    entry['repository'] = repo
    if username:
        entry['username'] = username
    if password:
        entry['password'] = password

    ppr.servers[server] = entry
    ppr.save()


@app.command()
def remove(server: str = '', rc: str = ''):
    ppr = PyPiRC(rc or None)
    if server:
        entry = ppr.servers.pop(server, None)
        if entry:
            ppr.save()
            sys.stdout.write('removed: %s' % entry)
            return 0
        else:
            sys.stderr.write(f'No server named {server}')
            return 1
    else:
        ppr.servers.clear()
        return 0



import typer
from typing_extensions import Annotated
from dektools.shell import shell_wrapper
from dektools.file import remove_path, write_file, read_text
from dektools.typer import command_mixin
from dektools.url import split_auth_url

app = typer.Typer(add_completion=False)


@app.command()
def config(
        update: Annotated[bool, typer.Option("--update/--no-update")] = False,
        cache: Annotated[bool, typer.Option("--cache/--no-cache")] = True,
        redirect: Annotated[bool, typer.Option("--redirect/--no-redirect")] = False,
):
    shell_wrapper(f"pdm config check_update {'true' if update else 'false'}")
    shell_wrapper(f"pdm config install.cache {'on' if cache else 'off'}")

    # disable mirror redirect
    from unearth import collector
    path_target = collector.__file__
    contents = (
        "        headers={",
        "        allow_redirects=False, headers={"
    )
    write_file(path_target, read_text(path_target).replace(*contents[::-1 if redirect else 1]))


@command_mixin(app)
def install(args):
    shell_wrapper(f'pdm install {args}')


@app.command()
def clear():
    from .core import get_pdm_cache_dir
    shell_wrapper(f'pdm cache clear')
    remove_path(get_pdm_cache_dir())


@app.command()
def clear_hash():
    from .core import get_pdm_cache_dir_hash
    remove_path(get_pdm_cache_dir_hash())


default_pypi_name = 'private'


@app.command()
def login(url, name: Annotated[str, typer.Argument()] = default_pypi_name):
    url, username, password = split_auth_url(url)
    shell_wrapper(f'pdm config pypi.{name}.url {url}')
    if url.startswith('https:'):
        shell_wrapper(f'pdm config pypi.{name}.verify_ssl true')
    shell_wrapper(f'pdm config pypi.{name}.username "{username}"')
    shell_wrapper(f'pdm config pypi.{name}.password "{password}"')


@app.command()
def logout(name: Annotated[str, typer.Argument()] = default_pypi_name):
    shell_wrapper(f'pdm config pypi.{name} --delete')



import os
import sys
import json
from dektools.sys import sys_paths_relative, path_sys_exe
from dektools.shell import shell_output


def get_pkg_entry_points(pkg, sys_exe=None):
    ev = (f"""print(__import__('json').dumps([x.name for x in __import__('importlib')."""
    f"""import_module('importlib.metadata').entry_points(name='{pkg}')]))""")
    if not sys_exe:
        sys_exe = sys.executable
    s = shell_output(f'{sys_exe} -c "{ev}"')
    try:
        return json.loads(s)
    except json.JSONDecodeError:
        raise ValueError(f"entry_points errors {pkg}: {s}") from None


def get_pkg_entry_points_paths(pkg, path_venv):
    path_venv_scripts = sys_paths_relative(path_venv)['scripts']
    path_venv_sys_exe = os.path.join(path_venv_scripts, os.path.basename(path_sys_exe()))
    entry_points = get_pkg_entry_points(pkg, path_venv_sys_exe)
    ext = os.path.splitext(path_venv_sys_exe)[1]
    return {
        entry_point: os.path.join(path_venv_scripts, f'{entry_point}{ext}') for entry_point in entry_points
    }



import json
import os
import subprocess
from pathlib import Path
from dektools.zip import decompress_files
from dektools.file import remove_path, sure_dir, read_text, write_file
from ..pypirc.core import PyPiRC


def get_pdm_cache_dir():
    return subprocess.getoutput('pdm config cache_dir').strip().split('\n', 1)[0].strip()


def get_pdm_cache_dir_hash():
    path = Path(get_pdm_cache_dir()) / 'hashes'
    assert str(path).startswith(os.path.expanduser('~'))
    return path


def get_pdm_cache_dir_pkg():
    path = Path(get_pdm_cache_dir()) / 'packages'
    assert str(path).startswith(os.path.expanduser('~'))
    return path


def get_pdm_cache_dir_metadata():
    path = Path(get_pdm_cache_dir()) / 'metadata'
    assert str(path).startswith(os.path.expanduser('~'))
    return path


def pdm_clear_cache_hash_pkg(server, pkg):
    from pdm.models.caches import HashCache
    path_cache = get_pdm_cache_dir_hash()
    hc = HashCache(path_cache)
    ppr = PyPiRC()
    repository = ppr.servers.get(server, {}).get('repository')
    if repository:
        url = f"{repository}/packages/{pkg}"
        path = hc._get_path_for_key(url)
        if str(path).startswith(str(path_cache)):
            remove_path(path_cache)


def pdm_clear_cache_hash():
    remove_path(get_pdm_cache_dir_hash())


def is_pdm_project(path):
    return os.path.isfile(os.path.join(path, 'pyproject.toml'))


def pdm_update_cache_pkg(path_dir, info):
    def handle_entry_points(fp):
        if os.path.isdir(fp):
            ep.save(os.path.join(fp, 'entry_points.txt'))

    name, version, ep = info['name'], info['version'], info['ep']
    prefix = f'{name}-{version}'
    path_pkgs = get_pdm_cache_dir_pkg()
    for fn in os.listdir(path_pkgs):
        if fn.startswith(prefix):
            path_pkg = os.path.join(path_pkgs, fn)
            path_lib = os.path.join(path_pkg, 'lib')
            fs = set(os.listdir(path_lib)) & set(os.listdir(path_dir))
            for f in fs:
                remove_path(os.path.join(path_lib, f), True)
                write_file(os.path.join(path_lib, f), ci=os.path.join(path_dir, f))

            for f in os.listdir(path_lib):
                if f.endswith('.dist-info'):
                    handle_entry_points(os.path.join(path_lib, f))
            for line in read_text(os.path.join(path_pkg, 'referrers')).splitlines():
                line = line.strip()
                if line:
                    handle_entry_points(line)


def pdm_clear_cache_pkg(pkg, dist=None):
    path_pkgs = get_pdm_cache_dir_pkg()
    for fn in os.listdir(path_pkgs):
        if dist:
            a = os.path.basename(dist).split('-')
            prefix = f'{a[0]}-{a[1]}'
            once = True
        else:
            prefix = f'{pkg}-'
            once = False
        if fn.startswith(prefix):
            path_pkg = os.path.join(path_pkgs, fn)
            if dist:
                path_lib = os.path.join(path_pkg, 'lib')
                remove_path(path_lib)
                sure_dir(path_lib)
                decompress_files(dist, path_lib)
            else:
                remove_path(path_pkg, True)
            if once:
                break


def pdm_clear_cache_metadata(dist):
    path_metadata = get_pdm_cache_dir_metadata()
    a = os.path.basename(dist).split('-')
    pkg, version = a[:2]
    for fn in os.listdir(path_metadata):
        path_meta = os.path.join(path_metadata, fn)
        data = json.loads(read_text(path_meta))
        for key in list(data.keys()):
            p, v = key.rsplit('-', 1)
            if (p.startswith(f'{pkg}[') or p == pkg) and v == version:
                data.pop(key, None)
        write_file(path_meta, json.dumps(data))



import os
import configparser
from urllib.parse import urlparse


class PipConf(object):
    CONF_FILE = os.path.join(os.path.expanduser('~'),
                             *(['pip', 'pip.ini'] if os.name == 'nt' else ['.pip', 'pip.conf']))
    CONF_KEYS = ['index-url', 'extra-index-url', 'trusted-host']

    def __init__(self, conf_file=None):
        if conf_file is None:
            self.conf_file = self.CONF_FILE
        else:
            self.conf_file = conf_file

        self.conf = configparser.ConfigParser()

        if os.path.exists(self.conf_file):
            self.conf.read(self.conf_file)

        self._create_global()

        self.data = self._conf2dict()

    def set_index_url(self, url, username=None, password=None):
        url = self._attach_auth(url, username, password)
        self.data[self.CONF_KEYS[0]] = [url]
        self._reset_trusted_host()
        self.save()

    def add_extra_url(self, url, username=None, password=None):
        url = self._attach_auth(url, username, password)
        self.data[self.CONF_KEYS[1]] = list({url, *self.data.get(self.CONF_KEYS[1], [])})
        self._reset_trusted_host()
        self.save()

    def remove_extra_url(self, url):
        new_lst = []
        new_lst_trusted = []
        old_lst = self.data.get(self.CONF_KEYS[1], [])
        pr = urlparse(url)
        for x in old_lst:
            pr_ = urlparse(x)
            if pr_.hostname != pr.hostname or pr_.path != pr.path:
                new_lst.append(x)
                new_lst_trusted.append(pr_.hostname)
        if new_lst:
            self.data[self.CONF_KEYS[1]] = new_lst
        else:
            self.data.pop(self.CONF_KEYS[1], None)
        self._reset_trusted_host()
        self.save()
        return len(new_lst) != len(old_lst)

    def clear_extra_url(self):
        self.data.pop(self.CONF_KEYS[1], None)
        self._reset_trusted_host()
        self.save()

    def _reset_trusted_host(self):
        self.data[self.CONF_KEYS[-1]] = list({urlparse(url).hostname for url in
                                              self.data.get(self.CONF_KEYS[0], []) + self.data.get(self.CONF_KEYS[1],
                                                                                                   [])})

    def _create_global(self):
        if not self.conf.has_section('global'):
            self.conf.add_section('global')

    def _reset_conf(self):
        if self.conf.has_section('global'):
            self.conf.remove_section('global')
        self._create_global()

    def _attach_auth(self, url, username, password):
        if username and password:
            pr = urlparse(url)
            return f'{pr.scheme}://{username}:{password}@{pr.hostname}:{pr.port}{pr.path}'
        return url

    def _conf2dict(self):
        data = {}
        for key in self.CONF_KEYS:
            value = self.conf.get('global', key, fallback=None)
            if value is not None:
                data[key] = [x.strip() for x in value.split('\n') if x.strip()]
        return data

    def _dict2conf(self, data):
        self._reset_conf()
        for key in self.CONF_KEYS:
            if key in data:
                self.conf.set('global', key, '\n\t'.join(data[key]))

    def save(self):
        self._dict2conf(self.data)
        dir_name = os.path.dirname(self.conf_file)
        if not os.path.isdir(dir_name):
            os.makedirs(dir_name)
        with open(self.conf_file, 'w') as configfile:
            self.conf.write(configfile)



from .main.command import app
from .pdm import command as pdm_command
from .pipconf import command as pipconf_command
from .pypirc import command as pypirc_command
from .script import command as script_command

app.add_typer(pdm_command.app, name='pdm')
app.add_typer(pipconf_command.app, name='pipconf')
app.add_typer(pypirc_command.app, name='pypirc')
app.add_typer(script_command.app, name='script')



import configparser
import os


class PyPiRC(object):
    """
    PyPi Configuration File Manager.

    Can be used for updating ~/.pypirc file programatically.

    Example::
        >>> a = PyPiRC('doctest_pypi.cfg')
        >>> new_server = {'pypi': {'repository': 'http://pypi.example.com', 'username':'test', 'password':'test'}}
        >>> new_server2 = {'pypi2': {'repository': 'http://pypi2.example.com'}}
        >>> a.servers.update(new_server)
        >>> a.servers.update(new_server2)
        >>> a.save()
        >>> 'pypi' in a.servers
        True
        >>> 'pypi2' in a.servers
        True
    """

    RC_FILE = os.path.join(os.path.expanduser('~'), '.pypirc')

    def __init__(self, rc_file=None):
        if rc_file is None:
            self.rc_file = self.RC_FILE
        else:
            self.rc_file = rc_file

        self.conf = configparser.ConfigParser()

        if os.path.exists(self.rc_file):
            self.conf.read(self.rc_file)

        self._create_distutils()

        self._servers = {}
        for server in self._get_index_servers():
            if self.conf.has_section(server):
                server_conf = {server: dict(self.conf.items(server))}
                self.servers.update(server_conf)

    def _create_distutils(self):
        """Creates top-level distutils stanza in pypirc."""
        if not self.conf.has_section('distutils'):
            self.conf.add_section('distutils')

    def save(self):
        """Saves pypirc file with new configuration information."""
        for server, conf in self.servers.items():
            self._add_index_server()
            for conf_k, conf_v in conf.items():
                if not self.conf.has_section(server):
                    self.conf.add_section(server)
                self.conf.set(server, conf_k, conf_v)

        dir_name = os.path.dirname(self.rc_file)
        if not os.path.isdir(dir_name):
            os.makedirs(dir_name)
        with open(self.rc_file, 'w') as configfile:
            self.conf.write(configfile)
        self.conf.read(self.rc_file)

    def _get_index_servers(self):
        """Gets index-servers current configured in pypirc."""
        idx_srvs = []
        if 'index-servers' in self.conf.options('distutils'):
            idx = self.conf.get('distutils', 'index-servers')
            idx_srvs = [srv.strip() for srv in idx.split('\n') if srv.strip()]
        return idx_srvs

    def _add_index_server(self):
        """Adds index-server to 'distutil's 'index-servers' param."""
        index_servers = '\n\t'.join(self.servers.keys())
        self.conf.set('distutils', 'index-servers', index_servers)

    @property
    def servers(self):
        """index-servers configured in pypirc."""
        return self._servers

    @servers.setter
    def servers(self, server):
        """Adds index-servers to pypirc."""
        self._servers.update(server)



import os
import sys
import typer
import virtualenv
from dektools.file import remove_path, write_file
from dektools.shell import shell_wrapper
from dektools.sys import sys_paths_relative
from ..active import activate_venv
from ..constants import dir_name_venv
from ..utils.common import get_py_pkg_name, get_py_pkg_info
from .core import get_pkg_path

app = typer.Typer(add_completion=False)


@app.command()
def create(name: str = dir_name_venv, path='.', parent=''):
    path = os.path.normpath(os.path.abspath(path))
    path_venv = os.path.join(path, name)
    if not os.path.exists(path_venv):
        virtualenv.cli_run([path_venv])
    if parent:
        path_parent = os.path.normpath(os.path.abspath(parent))
        path_var = sys_paths_relative('', True)
        path_lib = path_var['platlib']
        pth = f"import site;site.addsitedir(r'{path_parent}{path_lib}')"
        write_file(os.path.join(path_venv + path_lib, 'aaa__parent.pth'), s=pth)


@app.command()
def active(name: str = dir_name_venv):
    path_venv = os.path.join(os.getcwd(), name)
    activate_venv(path_venv)


@app.command()
def build(path: str = '.'):
    from ..pdm.core import is_pdm_project
    remove_path(os.path.join(path, '.pdm-build'))
    remove_path(os.path.join(path, 'build'))
    remove_path(os.path.join(path, 'dist'))
    if os.path.isdir(path):
        for fn in os.listdir(path):
            if fn.endswith('.egg-info'):
                remove_path(os.path.join(path, fn))

    if is_pdm_project(path):
        shell_wrapper(f'pdm build -p {path}')
    else:
        shell_wrapper(f'{sys.executable} -m build -s -w {path}')


@app.command()
def update(path_dir):
    from ..pdm.core import pdm_update_cache_pkg
    path_dir = os.path.normpath(os.path.abspath(path_dir))
    pdm_update_cache_pkg(path_dir, get_py_pkg_info(path_dir))


@app.command()
def push(server: str = '', path: str = '.', se: bool = True):
    from ..pdm.core import pdm_clear_cache_hash_pkg, pdm_clear_cache_pkg, pdm_clear_cache_metadata
    assert server
    build(path)
    path_pkg = get_pkg_path(path)
    pdm_clear_cache_hash_pkg(server, os.path.basename(path_pkg))
    pdm_clear_cache_pkg(get_py_pkg_name(path), path_pkg)
    pdm_clear_cache_metadata(path_pkg)
    shell_wrapper(f'twine upload {"--skip-existing" if se else ""} --repository {server} {path_pkg}')


@app.command()
def var(name):
    data = sys_paths_relative('', True)
    if name:
        v = data[name]
    else:
        v = data
    sys.stdout.write(str(v))
    sys.stdout.flush()



import os
import json
import time
import virtualenv
import typer
import typing_extensions
from packaging.requirements import Requirement
from dektools.shell import shell_wrapper
from dektools.sys import sys_paths_relative, path_sys_exe
from dektools.file import write_file, read_text, sure_dir, remove_path
from ..constants import path_work_dir, dir_name_venv
from ..utils.common import get_py_pkg_name
from ..main.command import build, get_pkg_path
from .core import get_pkg_entry_points_paths

path_scripts = os.path.join(path_work_dir, 'scripts')
path_scripts_info = os.path.join(path_work_dir, '.scripts.json')

path_pyproject = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'res', 'pyproject.toml-tpl')

app = typer.Typer(add_completion=False)


@app.command()
def add(pkg: typing_extensions.Annotated[str, typer.Argument()] = '', name=None, path='.'):
    from ..pdm.core import is_pdm_project
    if name and not pkg:
        pkg = name
    if pkg:
        req = Requirement(pkg)
        if not name:
            name = req.name
    else:
        assert path
        path = os.path.normpath(os.path.abspath(path))
        if not name:
            name = get_py_pkg_name(path)
        if os.path.isdir(path):
            if not is_pdm_project(path):
                build(path)
    path_scripts_item = os.path.join(path_scripts, name)
    sure_dir(path_scripts_item)
    path_venv = os.path.join(path_scripts_item, dir_name_venv)
    virtualenv.cli_run([path_venv])
    path_venv_scripts = sys_paths_relative(path_venv)['scripts']
    bin_exe_pre = set(os.listdir(path_venv_scripts))
    path_pyproject_item = os.path.join(path_scripts_item, 'pyproject.toml')
    if not os.path.exists(path_pyproject_item):
        write_file(path_pyproject_item, s=read_text(path_pyproject))
    os.system(f'pdm lock -p {path_scripts_item}')
    command_record = 'dekvenv add '
    if pkg:
        cmd = f'pdm add "{pkg}" -v -p {path_scripts_item}'
        command_record += f'--pkg {pkg}'
    else:
        if os.path.isfile(path):
            file = path
        elif is_pdm_project(path):
            file = path
        else:
            file = get_pkg_path(path)
        cmd = f'pdm add file:///{file} -v -p {path_scripts_item}'
        command_record += f'--path {file}'
    shell_wrapper(cmd)
    entry_points = get_pkg_entry_points_paths(name, path_venv).values()
    while True:
        for ep in entry_points:
            if not os.path.isfile(ep):
                time.sleep(0)
                continue
        break
    bin_exe_set = set(os.listdir(path_venv_scripts)) - bin_exe_pre
    path_sys_exe_scripts = sys_paths_relative(os.path.dirname(path_sys_exe()))['scripts']
    dict_path_cmd_proxy = {}
    for bin_exe in bin_exe_set:
        path_bin_exe_real = os.path.join(path_venv_scripts, bin_exe)
        path_exe_proxy = os.path.join(path_sys_exe_scripts, bin_exe)
        bin_write(name, path_bin_exe_real, path_exe_proxy)
        dict_path_cmd_proxy[bin_exe] = path_exe_proxy
    scripts_info = json.loads(read_text(path_scripts_info, '{}'))
    scripts_info.setdefault('path', {})[name] = path_scripts_item
    record_info = scripts_info.setdefault('record', {})
    record_info_list = record_info.setdefault(name, [])
    if command_record not in record_info_list:
        record_info_list.append(command_record)
    dict_info = scripts_info.setdefault('proxy', {})
    dict_info = dict_info.setdefault(name, {})
    dict_info.update(dict_path_cmd_proxy)
    write_file(path_scripts_info, json.dumps(scripts_info, indent=4, ensure_ascii=False))


path_bin_work = sys_paths_relative(os.path.dirname(path_sys_exe()))['scripts']
path_bin_proxy = os.path.join(path_bin_work, '.dekvenv', 'bin')


def bin_write(name, path_bin_exe_real, path_exe_proxy):
    write_file(
        os.path.join(
            path_bin_proxy, '.dekvenv', 'bin', os.path.basename(path_exe_proxy), name),
        c=path_bin_exe_real
    )
    ok = remove_path(path_exe_proxy, True)
    if ok:
        write_file(
            path_exe_proxy,
            c=path_bin_exe_real
        )
        if os.name != 'nt':
            os.system(f'chmod +x {path_exe_proxy}')


def bin_remove(name):
    bin_list = set()
    if os.path.isdir(path_bin_proxy):
        for bin_name in os.listdir(path_bin_proxy):
            bin_file = os.path.join(path_bin_proxy, bin_name, name)
            if os.path.exists(bin_file):
                remove_path(bin_file)
                bin_list.add(bin_name)
    for bin_name in bin_list:
        ok = remove_path(os.path.join(path_bin_proxy, bin_name), True)
        if ok:
            ret = os.listdir(os.path.join(path_bin_proxy, bin_name))
            if ret:
                write_file(
                    os.path.join(path_bin_work, bin_name),
                    c=os.path.join(path_bin_proxy, bin_name, ret[0])
                )
                if os.name != 'nt':
                    os.system(f'chmod +x {os.path.join(path_bin_work, bin_name)}')


def get_cmd_proxy_ref(scripts_info):
    dict_proxy = scripts_info.get('proxy') or {}
    result = {}
    for d_cmd in dict_proxy.values():
        for proxy in d_cmd:
            result[proxy] = result.get(proxy, 0) + 1
    return result


@app.command()
def remove(name):
    scripts_info = json.loads(read_text(path_scripts_info, '{}'))

    dict_path = scripts_info.get('path') or {}
    path_scripts_item = dict_path.get(name)
    if path_scripts_item:
        remove_path(path_scripts_item, True)

    cmd_proxy_ref = get_cmd_proxy_ref(scripts_info)
    record_info = scripts_info.get('record', {})
    dict_proxy = scripts_info.get('proxy') or {}
    dict_proxy_name = dict_proxy.get(name) or {}
    for cmd_proxy, path_cmd_proxy in dict_proxy_name.items():
        if cmd_proxy_ref.get(cmd_proxy, 0) <= 1:
            remove_path(path_cmd_proxy, True)

    record_info.pop(name, None)
    dict_path.pop(name, None)
    dict_proxy.pop(name, None)
    bin_remove(name)

    write_file(path_scripts_info, json.dumps(scripts_info, indent=4, ensure_ascii=False))


@app.command()
def reset(name):
    remove(name)
    add(name)


@app.command()
def pop(name: str = '', pkg: str = ''):
    assert pkg and name
    path_scripts_item = os.path.join(path_scripts, name)
    if os.path.exists(path_scripts_item):
        shell_wrapper(f'pdm remove "{pkg}" -v -p {path_scripts_item}')
    else:
        raise


@app.command()
def record():
    scripts_info = json.loads(read_text(path_scripts_info, '{}'))
    record_info = scripts_info.get('record', {})
    for name in sorted(record_info.keys()):
        array = record_info[name]
        for item in array:
            print(item)


@app.command()
def clean():
    scripts_info = json.loads(read_text(path_scripts_info, '{}'))
    dict_path = scripts_info.get('path') or {}
    for name in dict_path.keys():
        remove(name)
    remove_path(path_scripts_info)


@app.command()
def info():
    print(json.dumps(json.loads(read_text(path_scripts_info, '{}')), indent=4, ensure_ascii=False))



import os


def get_pkg_path(path):
    path_dist = os.path.join(path, 'dist')
    path_pkg = os.path.join(path_dist, next(x for x in os.listdir(path_dist) if x.endswith('.whl')))
    return path_pkg



import os
import configparser
import toml
from dektools.file import read_text, write_file, remove_path


def get_py_pkg_info(path_dir):
    if os.path.isfile(path_dir) and os.path.splitext(path_dir)[-1] == '.whl':
        return os.path.basename(path_dir).split('-', 1)[0]
    path_cfg = os.path.join(path_dir, 'setup.cfg')
    path_toml = os.path.join(path_dir, 'pyproject.toml')
    result = {}
    if os.path.isfile(path_cfg):
        conf = configparser.ConfigParser()
        conf.read(path_cfg)
        result.update(
            name=conf.get('metadata', 'name', fallback=None),
            version=conf.get('metadata', 'version', fallback=None),
            ep=EntryPoints().from_conf(conf)
        )
    elif os.path.isfile(path_toml):
        conf = toml.loads(read_text(path_toml))
        result.update(
            name=conf['project']['name'] or None,
            version=conf['project']['version'] or None,
            ep=EntryPoints().from_json(conf)
        )
    return result


def get_py_pkg_name(path_dir):
    return get_py_pkg_info(path_dir)['name']


class EntryPoints:
    def __init__(self):
        self.data = []

    def save(self, fp):
        s = '\n'.join(self.data)
        remove_path(fp)
        if s.strip():
            write_file(fp, s=s)

    def from_conf(self, conf):
        section = 'options.entry_points'
        if conf.has_section(section):
            for option in conf.options(section):
                self.data.append(f'[{option}]')
                d = conf.get(section, option, fallback=None),
                self.data.extend(d)
        return self

    def from_json(self, data):
        scripts = (data.get('project') or {}).get('scripts')
        if scripts:
            self.data.append('[console_scripts]')
            for k, v in scripts.items():
                self.data.append(f'\n{k} = {v}')

        scripts = (data.get('project') or {}).get('gui-scripts')
        if scripts:
            self.data.append('[gui_scripts]')
            for k, v in scripts.items():
                self.data.append(f'\n{k} = {v}')

        eps = (data.get('project') or {}).get('entry-points')
        if eps:
            for name, d in eps.items():
                self.data.append(f'[{name}]')
                for k, v in d.items():
                    self.data.append(f'\n{k} = {v}')
        return self



import os
import sys
from dektools.file import read_text
from dektools.sys import sys_paths_relative
from .constants import dir_name_venv


def activate_venv(path_venv=None, ignore=False):
    path_venv = path_venv or dir_name_venv
    path_scripts = sys_paths_relative(path_venv)['scripts']
    this_file = os.path.join(path_scripts, 'activate_this.py')
    if not ignore or os.path.isfile(this_file):
        exec(read_text(this_file), {'__file__': this_file})


def is_venv_active(sp=None):
    return (sp or sys.prefix) != sys.base_prefix
